@@ -3,15 +3,11 @@ use strict;
 use warnings;
 #use Pugs::Compiler::Rule;
 use Pugs::Compiler::Regex;
+use Pugs::Compiler::Token;
 use base qw(Pugs::Grammar::Base);
 use Data::Dumper;
 
-sub ws {
-    my $grammar = shift;
-    $_[0] = "" unless defined $_[0];
-    my $bool = $_[0] =~ /
-        ^
-        (
+*ws = Pugs::Compiler::Regex->compile( '
             (?:
                 # pod start
                 (?: \n | ^ ) \= \w  
@@ -27,39 +23,27 @@ sub ws {
                 # a comment until end-of-line
                 \# (?-s:.)*   
             )+
-        )
-        (.*)$   # "tail"
-    /sx;
-    #print "ws: $grammar [$_[0]] -- [$2] \n";
-    return {
-        bool  => $bool,
-        match => $1,
-        tail  => $2,
-    }
-};
+', 
+    { Perl5 => 1 } 
+)->code;
     
 sub add_rule {
     my ( $class, $key, $rule ) = @_;
     no strict qw( refs );
-    ${"${class}::hash"}{$key} = Pugs::Compiler::Regex->compile( 
-        $rule, 
-        { grammar => $class },
-    );
-}
-
-sub capture {
-    #print "capture1: ", Dumper( $_[0] ); 
-    #print "capture2: ", Dumper( $_[0]->data->{match}[0]{match}[1]{capture} ); 
-    return $_[0]->data->{match}[0]{match}[1]{capture};
+    #print "add_rule [". $class . "::hash{$key} /$rule/\n";
+    ${ $class . "::hash" }{$key} = $rule;
 }
 
 sub recompile {
     my ( $class ) = @_;
     no strict qw( refs );
     #print "creating ${class}::parse()\n";
-    *{"${class}::parse"} = Pugs::Compiler::Regex->compile( '
-        %' . $class . '::hash
-        { return Pugs::Grammar::BaseCategory::capture( $/ ) }
+    *{"${class}::parse"} = Pugs::Compiler::Token->compile( '
+        <%' . $class . '::hash>
+        { 
+            #print "BaseCategory matched hash ", Dumper( $_[0]->data );
+            return $/->{\'' . $class . '::hash\'}->();
+        }
     ' )->code;
 }
 
