@@ -16,25 +16,21 @@ use Data::Dumper;
 # *ws = &Pugs::Grammar::BaseCategory::ws;
 
 sub perl6_expression {
-    my $class = shift;
-    my $src   = shift;
-    my $param = shift;
-
-    #warn "perl6_expression param: ", Dumper @_;
-
-    my ( $ast, $tail ) = Pugs::Grammar::Expression::ast( $src, $param );
-    if ( length( $tail ) ) {
-        $src = substr( $src, 0, - length( $tail ) );
-    }
-    return Pugs::Runtime::Match->new( { 
-        bool  =>   ( $ast ? 1 : 0 ),
-        match =>   $src,
-        tail  =>   $tail,
-        capture => $ast,
-    } )
+    #print "perl6_expression param: ", Dumper @_;
+    my $pos = $_[2]{p} || 0;
+    my ( $ast, $to ) = Pugs::Grammar::Expression::ast( $_[1], $_[2] );
+    my $match = Pugs::Runtime::Match->new( { 
+        bool    => \( $ast ? 1 : 0 ),
+        str     => \$_[1],
+        match   => [],
+        from    => \$pos,
+        to      => \$to,
+        capture => \$ast,
+    } );
+    return $match;
 };
 
-*perl6_expression_or_null = Pugs::Compiler::Regex->compile( q(
+*perl6_expression_or_null = Pugs::Compiler::Token->compile( q(
     <perl6_expression('no_blocks',0)> 
         { return $_[0]{perl6_expression}->() }
     |
@@ -45,25 +41,38 @@ sub perl6_expression {
     { grammar => __PACKAGE__ }
 )->code;
 
-*block = Pugs::Compiler::Regex->compile( q(
-    \{ : <?ws>? <statements_or_null> <?ws>? \}
-        { return { 
-            bare_block => $_[0]{statements_or_null}->(),
-        } }
+*block = Pugs::Compiler::Token->compile( q(
+    \{ <?ws>? 
+        #{ print "block\n" }
+        <statements>
+        #{ print "matched block\n" }
+        <?ws>? \}
+        { 
+            return { 
+                bare_block => $_[0]{statements}->(),
+            } 
+        }
     |
-    \-\> : 
+    \{ <?ws>? \}
+        { 
+            return { 
+                bare_block => { statements => [] } 
+            } 
+        }
+    |
+    \-\>  
         [
             <?ws>? <signature_no_invocant> <?ws>? 
-            \{ <?ws>? <statements_or_null> <?ws>? \}
+            \{ <?ws>? <statements> <?ws>? \}
             { return { 
-                pointy_block => $_[0]{statements_or_null}->(),
+                pointy_block => $_[0]{statements}->(),
                 signature    => $_[0]{signature_no_invocant}->(),
             } }
         |
             <?ws>?
-            \{ <?ws>? <statements_or_null> <?ws>? \}
+            \{ <?ws>? <statements> <?ws>? \}
             { return { 
-                pointy_block => $_[0]{statements_or_null}->(),
+                pointy_block => $_[0]{statements}->(),
                 signature    => undef,
             } }
         ]
@@ -71,127 +80,8 @@ sub perl6_expression {
     { grammar => __PACKAGE__ }
 )->code;
 
-
-*if = Pugs::Compiler::Regex->compile( q(
-    (if|unless) : <?ws>? 
-    $<exp1> := <perl6_expression('no_blocks',0)> <?ws>?
-    $<exp2> := <block>        
-        [
-            <?ws>? else <?ws>? 
-            $<exp3> := <block>
-                { return { 
-                    statement => $_[0][0]->(),
-                    exp1 => $_[0]{exp1}->(),
-                    exp2 => $_[0]{exp2}->(),
-                    exp3 => $_[0]{exp3}->(),
-                } }
-        |
-            <?ws>? elsif <?ws>? 
-            $<exp3> := <perl6_expression('no_blocks',0)> <?ws>?
-            $<exp4> := <block>
-            [
-                <?ws>? else <?ws>? 
-                $<exp5> := <block>
-                { return { 
-                    statement => $_[0][0]->(),
-                    exp1 => $_[0]{exp1}->(),
-                    exp2 => $_[0]{exp2}->(),
-                    exp3 => $_[0]{exp3}->(),
-                    exp4 => $_[0]{exp4}->(),
-                    exp5 => $_[0]{exp5}->(),
-                } }
-                
-                # TODO: elsif ...
-            |
-                { return { 
-                    statement => $_[0][0]->(),
-                    exp1 => $_[0]{exp1}->(),
-                    exp2 => $_[0]{exp2}->(),
-                    exp3 => $_[0]{exp3}->(),
-                    exp4 => $_[0]{exp4}->(),
-                } }
-            ]
-        |
-            { return { 
-                    statement => $_[0][0]->(),
-                    exp1 => $_[0]{exp1}->(),
-                    exp2 => $_[0]{exp2}->(),
-            } }
-        ]
-),
-    { grammar => __PACKAGE__ }
-)->code;
-
-
-*for = Pugs::Compiler::Regex->compile( q(
-    (for|while|until) : <?ws>? 
-    $<exp1> := <perl6_expression('no_blocks',0)> <?ws>?
-    $<exp2> := <block>        
-        { return { 
-                    statement => $_[0][0]->(),
-                    exp1 => $_[0]{exp1}->(),
-                    exp2 => $_[0]{exp2}->(),
-        } }
-),
-    { grammar => __PACKAGE__ }
-)->code;
-
-*loop = Pugs::Compiler::Regex->compile( q(
-    (loop) : <?ws>?
-        [
-          <'('> $<exp1> := <perl6_expression_or_null> <?ws>? <';'>
-                $<exp2> := <perl6_expression_or_null> <?ws>? <';'>
-                $<exp3> := <perl6_expression_or_null> <?ws>? <')'> 
-                <?ws>? $<content> := <block>
-          { return { statement => $_[0][0]->(),
-                     exp1      => $_[0]{exp1}->(),
-                     exp2      => $_[0]{exp2}->(),
-                     exp3      => $_[0]{exp3}->(),
-                     content   => $_[0]{content}->() }
-          }
-        |
-          <block> <?ws>?
-          { return { statement => $_[0][0]->(),
-                     content   => $_[0]{block}->() }
-          }
-        |
-            # XXX better error messages
-            { return { die "invalid loop syntax" } }
-       ]
-),
-    { grammar => __PACKAGE__ }
-)->code;
-
-*repeat = Pugs::Compiler::Regex->compile( q(
-    (repeat) : <?ws>?
-        [
-          (while|until) : <?ws>? <perl6_expression('no_blocks',0)>
-          <block> <?ws>?
-          { return { statement => $_[0][0]->(),
-                     which     => $_[0][1]->(),
-                     exp2      => $_[0]{perl6_expression}->(),
-                     postfix   => 1,
-                     content   => $_[0]{block}->() }
-          }
-        |
-          <block> <?ws>?
-          (while|until) : <?ws>? <perl6_expression('no_blocks',0)>
-          { return { statement => $_[0][0]->(),
-                     which     => $_[0][1]->(),
-                     exp2      => $_[0]{perl6_expression}->(),
-                     postfix   => 1,
-                     content   => $_[0]{block}->() }
-          }
-        |
-            # XXX better error messages
-            { return { die "invalid repeat syntax" } }
-       ]
-),
-    { grammar => __PACKAGE__ }
-)->code;
-
-*try = Pugs::Compiler::Regex->compile( q(
-    (try) : <?ws>? <block>        
+*try = Pugs::Compiler::Token->compile( q(
+    (try) <?ws>? <block>        
         { return { 
                     fixity => 'prefix',
                     op1 => { op => 'try' },
@@ -233,16 +123,16 @@ sub perl6_expression {
 
 # Str|Num
 *signature_term_type = Pugs::Compiler::Regex->compile( q(
-        (<alnum>+)
+        <Pugs::Grammar::Term.bare_ident>
         [
             <?ws>? \| <?ws>? <signature_term_type>
             { return {
                 op1 => '|',
-                exp1 => $_[0][0]->(), 
+                exp1 => $_[0]{'Pugs::Grammar::Term.bare_ident'}->(),
                 exp2 => $<signature_term_type>->(),
             } }
         |
-            { return $_[0][0]->() }
+            { return $_[0]{'Pugs::Grammar::Term.bare_ident'}->() }
         ]
     |
         { return undef }
@@ -264,7 +154,7 @@ sub perl6_expression {
 
 
 *signature_term = Pugs::Compiler::Regex->compile( q(
-        <signature_term_type> <?ws>?
+        <signature_term_type> : <?ws>?
         (<':'>?)
         (<'*'>?)
         <signature_term_ident>
@@ -306,7 +196,7 @@ sub perl6_expression {
 )->code;
 
 
-*signature = Pugs::Compiler::Regex->compile( q(
+*signature = Pugs::Compiler::Token->compile( q(
         <signature_term> <?ws>? <':'>
         [
             <?ws>? <signature_no_invocant>
@@ -333,7 +223,7 @@ sub perl6_expression {
     { grammar => __PACKAGE__ }
 )->code;
 
-*sub_decl_name = Pugs::Compiler::Regex->compile( q(
+*sub_decl_name = Pugs::Compiler::Token->compile( q(
     ( my | <''> ) <?ws>?
     ( multi | <''> ) <?ws>?
     ( submethod | method | sub ) <?ws>? 
@@ -357,18 +247,20 @@ sub perl6_expression {
     { grammar => __PACKAGE__ }
 )->code;
 
-*sub_signature = Pugs::Compiler::Regex->compile( q(
+*sub_signature = Pugs::Compiler::Token->compile( q(
         # (sig)
-        <'('> : <?ws>? <signature> <?ws>? <')'>
-        { return $_[0]{signature}->() }
+        <'('> <?ws>? <signature> <?ws>? <')'>
+        { 
+            #print "sig ", Dumper( $_[0]{signature}->() );
+            return $_[0]{signature}->() 
+        }
     |
         { return [] }
 ),
     { grammar => __PACKAGE__ }
 )->code;
 
-
-*sub_decl = Pugs::Compiler::Regex->compile( q(
+*sub_decl = Pugs::Compiler::Token->compile( q(
     <sub_decl_name> <?ws>? 
         # (sig)
         <sub_signature> <?ws>? 
@@ -478,7 +370,7 @@ sub perl6_expression {
 # /class
 
 
-*begin_block = Pugs::Compiler::Regex->compile( q(
+*begin_block = Pugs::Compiler::Token->compile( q(
     (          
    BEGIN 
  | CHECK 
@@ -495,7 +387,7 @@ sub perl6_expression {
  | POST
  | CATCH
  | CONTROL
-    ) : <?ws>? <block>        
+    ) <?ws>? <block>        
         { return { 
             trait  => $_[0][0]->(),
             %{ $_[0]{block}->() },
@@ -504,42 +396,38 @@ sub perl6_expression {
     { grammar => __PACKAGE__ }
 )->code;
 
-
-*statement = Pugs::Compiler::Regex->compile( q(
-    use <?ws> v5 <?ws>?; ((.)*?) ; <?ws>? use <?ws> v6 (.)*? ; 
+*perl5source = Pugs::Compiler::Regex->compile( q(
+    (.*?) ; <?ws>? use <?ws> v6 (.)*? ; 
         { return { 
             perl5source => $_[0][0]->() 
         } }
+),
+    { grammar => __PACKAGE__ }
+)->code;
+
+*statement = Pugs::Compiler::Token->compile( q(
+    use <?ws> v5 <?ws>?; <perl5source> 
+        { return $_[0]{perl5source}->() 
+        }
+    |
+    <Pugs::Grammar::StatementControl.parse>
+        { 
+            return $/->{'Pugs::Grammar::StatementControl.parse'}->();
+        }
     |
     <begin_block>
         { return $_[0]{begin_block}->();
         }
     |
-    <rule_decl>
+    <rule_decl>  # TODO: move to expression
         { return $_[0]{rule_decl}->();
         }
     |
-    <sub_decl>
+    <sub_decl>   # TODO: move to expression
         { return $_[0]{sub_decl}->();
         }
     |
-    <if>
-        { return $_[0]{if}->();
-        }
-    |
-    <for>
-        { return $_[0]{for}->();
-        }
-    |
-    <loop>
-        { return $_[0]{loop}->();
-        }
-    |
-    <repeat>
-        { return $_[0]{repeat}->();
-        }
-    |
-    <try>   # this actually don't belong here
+    <try>        # TODO: move to expression
         { return $_[0]{try}->();
         }
     |
@@ -547,13 +435,18 @@ sub perl6_expression {
         [
             <?ws>? (if|unless|for|while|until) <?ws>?
             $<exp1> := <perl6_expression> 
+            #{ print "$a if $b ", Dumper( $/->data );
+            #    print Dumper( $_[0]{perl6_expression}->data ),
+            #          Dumper( $_[0]{exp1}->data );
+            #}
             { return {
                 statement => $_[0][0]->(),
                 exp2 => $_[0]{perl6_expression}->(),
                 exp1 => $_[0]{exp1}->(),
             } } 
         |
-            { return $_[0]{perl6_expression}->();
+            { 
+                return $_[0]{perl6_expression}->();
             } 
         ]
     |
@@ -565,12 +458,17 @@ sub perl6_expression {
     { grammar => __PACKAGE__ }
 )->code;
 
-*statements = Pugs::Compiler::Regex->compile( q(
+*statements = Pugs::Compiler::Token->compile( q(
     [ ; <?ws>? ]*
+
     [
-        <statement> :
+        <before <'}'> > { $::_V6_SUCCEED = 0 } 
+    |
+        <statement> 
+        <?ws>? [ ; <?ws>? ]*
         [
-            <?ws>? [ ; <?ws>? ]*
+            <before <'}'> > { $::_V6_SUCCEED = 0 }
+        |
             <statements> 
             { return {
                 statements => [
@@ -580,35 +478,21 @@ sub perl6_expression {
                 }
             }
         |
-            { return {
+            { 
+            return {
                 statements => [ $_[0]{statement}->() ],
             } }
         ]
-    |
-        { return {
-            statements => [],
-        } }
     ]
 ),
     { grammar => __PACKAGE__ }
 )->code;
 
-*statements_or_null = Pugs::Compiler::Regex->compile( q(
+*parse = Pugs::Compiler::Token->compile( q(
+    <?ws>? 
     <statements> 
+    <?ws>? 
         { return $_[0]{statements}->() }
-    |
-        { return {
-            statements => [],
-        } }
-),
-    { grammar => __PACKAGE__ }
-)->code;
-
-*parse = Pugs::Compiler::Regex->compile( q(
-    <?ws>? 
-    <statements_or_null> 
-    <?ws>? 
-        { return $_[0]{statements_or_null}->() }
 ),
     { grammar => __PACKAGE__ }
 )->code;
